/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CAENCfdController.h"
#include "drivers/global/DriversCommands.h"
#include "controllers/common/family/Families.h"
#include <boost/thread/thread.hpp>
#include <lstdpp128/lstdpp.h>
#include <controllers/common/acquisition/mode/DPPAcquisitionCommon.h>

using namespace std;
using namespace boost;

namespace npp {

const string CAENCfdController::TYPE = "cfd_controller";

/*
 * Constructor
 */
CAENCfdController::CAENCfdController(const string& name) : CAENAdcController(name) {

	setFamily(family::ACQUISITION, family::SETTING);

	// Register properties
	// CFD parameters
	delay.init(this, NOSAVE, "delay");
	attenuation.init(this, NOSAVE, "attenuation");

	// Energy filter parameters
	trapezoidRise.init(this, NOSAVE, "trapezoid_rise");
	trapezoidFlat.init(this, NOSAVE, "trapezoid_flat");
	decayTime.init(this, NOSAVE, "decay_time");
	peaking.init(this, NOSAVE, "peaking");
	bitShift.init(this, NOSAVE, "bit_shift");
	channelOffset.init(this, SAVE, "channel_offset");

	m_CfdDriver.init(this, "cfd_driver");

	registerFunction(TYPE);
}

/*
 * Destructor
 */
CAENCfdController::~CAENCfdController() {

}

/*
 * postConfiguration
 */
void CAENCfdController::postConfiguration() {
	registerRefresher(recordLength, &CAENCfdController::refreshRecordLengthProperty, this);
	registerRefresher(nbChannels, &CAENCfdController::refreshNbChannelsProperty, this);
	registerRefresher(dppAcqMode, &CAENCfdController::refreshDppAcqModeProperty, this);
	registerRefresher(displayChannel, &CAENCfdController::refreshDisplayChannelProperty, this);
	registerRefresher(channelActive, &CAENCfdController::refreshChannelActiveProperty, this);
	registerRefresher(pulsePolarity, &CAENCfdController::refreshPulsePolarityProperty, this);
	registerRefresher(delay, &CAENCfdController::refreshDelayProperty, this);
	registerRefresher(attenuation, &CAENCfdController::refreshAttenuationProperty, this);
	registerRefresher(pulsePolarity, &CAENCfdController::refreshPulsePolarityProperty, this);
	registerRefresher(trapezoidRise, &CAENCfdController::refreshTrapezoidRiseProperty, this);
	registerRefresher(trapezoidFlat, &CAENCfdController::refreshTrapezoidFlatProperty, this);
	registerRefresher(decayTime, &CAENCfdController::refreshDecayTimeProperty, this);
	registerRefresher(peaking, &CAENCfdController::refreshPeakingProperty, this);
	registerRefresher(bitShift, &CAENCfdController::refreshBitShiftProperty, this);
	registerRefresher(channelOffset, &CAENCfdController::refreshChannelOffsetProperty, this);

	updateBoardProperties();
	updateCfdProperties(displayChannel() + channelOffset());
	cardType = lstdpp128::CFD;
}

void CAENCfdController::refreshDelayProperty(int32 aValue) throw (CannotSetValue) {
	cout << "CAENCfdController::refreshDelayProperty" << endl;
	m_CfdDriver->delay = aValue;
	boost::thread td(boost::bind(&CAENCfdController::writeParam, this));
}

void CAENCfdController::refreshAttenuationProperty(int32 aValue) throw (CannotSetValue) {
	cout << "CAENCfdController::refreshAttenuationProperty" << endl;
	m_CfdDriver->attenuation = aValue;
	boost::thread td(boost::bind(&CAENCfdController::writeParam, this));
}

void CAENCfdController::refreshPulsePolarityProperty(int32 aValue) throw (CannotSetValue) {
	cout << "CAENCfdController::refreshPulsePolarityProperty" << endl;
	m_CfdDriver->pulsePolarity.set(displayChannel() + channelOffset(), aValue);
	boost::thread td(boost::bind(&CAENCfdController::writeParam, this));
}

void CAENCfdController::refreshTrapezoidRiseProperty(int32 aValue) throw (CannotSetValue) {
	cout << "CAENCfdController::refreshTrapezoidRiseProperty" << endl;
	m_CfdDriver->trapezoidRise.set(displayChannel() + channelOffset(), aValue);
	boost::thread td(boost::bind(&CAENCfdController::writeParam, this));
}

void CAENCfdController::refreshTrapezoidFlatProperty(int32 aValue) throw (CannotSetValue) {
	cout << "CAENCfdController::refreshTrapezoidFlatProperty" << endl;
	m_CfdDriver->trapezoidFlat.set(displayChannel() + channelOffset(), aValue);
	boost::thread td(boost::bind(&CAENCfdController::writeParam, this));
}

void CAENCfdController::refreshDecayTimeProperty(int32 aValue) throw (CannotSetValue) {
	cout << "CAENCfdController::refreshDecayTimeProperty" << endl;
	m_CfdDriver->decayTime.set(displayChannel() + channelOffset(), aValue);
	boost::thread td(boost::bind(&CAENCfdController::writeParam, this));
}

void CAENCfdController::refreshPeakingProperty(int32 aValue) throw (CannotSetValue) {
	cout << "CAENCfdController::refreshPeakingProperty" << endl;
	m_CfdDriver->peaking.set(displayChannel() + channelOffset(), aValue);
	boost::thread td(boost::bind(&CAENCfdController::writeParam, this));
}

void CAENCfdController::refreshBitShiftProperty(int32 aValue) throw (CannotSetValue) {
	cout << "CAENCfdController::refreshBitShiftProperty" << endl;
	m_CfdDriver->bitShift.set(displayChannel() + channelOffset(), aValue);
	boost::thread td(boost::bind(&CAENCfdController::writeParam, this));
}

void CAENCfdController::refreshDisplayChannelProperty(int32 aChannel) throw (CannotSetValue) {
	updateCfdProperties(aChannel + channelOffset());
}

void CAENCfdController::refreshChannelActiveProperty(int32 aValue) throw (CannotSetValue) {
	cout << "CAENCfdController::refreshChannelActiveProperty" << endl;
	m_CfdDriver->channelActive.set(displayChannel(), aValue);
	boost::thread td(boost::bind(&CAENCfdController::writeParam, this));
}

void CAENCfdController::refreshChannelOffsetProperty(int32 aChannel) throw (CannotSetValue) {
	cout << "CAENCfdController::refreshChannelOffsetProperty" << endl;
	updateCfdProperties(displayChannel() + aChannel);
	boost::thread td(boost::bind(&CAENCfdController::writeParam, this));
}

void CAENCfdController::refreshNbChannelsProperty(int32 aValue)  throw (CannotSetValue) {
	minRange = 0;
	maxRange = aValue - 1;
	m_CfdDriver->nbChannels = aValue;
	if (displayChannel() >= aValue)
		displayChannel = aValue - 1;
}

void CAENCfdController::refreshDppAcqModeProperty(int32 aValue)  throw (CannotSetValue) {
	cout << "CAENCfdController::refreshDppAcqModeProperty" << endl;
	if (aValue == acquisition::DPPAcquisitionCommon::SCOPE_MODE) {
		m_CfdDriver->dppAcqMode = 1;
	}
	else {
		m_CfdDriver->dppAcqMode = 0;
	}
	boost::thread td(boost::bind(&CAENCfdController::writeParam, this));
}

void CAENCfdController::refreshRecordLengthProperty(int32 aValue) throw (CannotSetValue) {

//	m_CfdDriver->recordLength = aValue;
}

/*
 * updateBoardProperties
 */
void CAENCfdController::updateBoardProperties() {
//	modelName.update(m_CfdDriver->modelName());
	channels.update(m_CfdDriver->channels());
	nbChannels.update(m_CfdDriver->nbChannels());
	minRange = 0;
	maxRange = m_CfdDriver->nbChannels() - 1;
	dppAcqMode.update(m_CfdDriver->dppAcqMode());
	delay.update(m_CfdDriver->delay());
	attenuation.update(m_CfdDriver->attenuation());
}

/*
 * updateCfdProperties
 */
void CAENCfdController::updateCfdProperties(int32 channel) {
	cout << "updateCfdProperties : " << channel << endl;
	m_CfdDriver->scopeChannel = channel;
	channelActive.update(m_CfdDriver->channelActive.get(channel));
	pulsePolarity.update(m_CfdDriver->pulsePolarity.get(channel));
	trapezoidRise.update(m_CfdDriver->trapezoidRise.get(channel));
	trapezoidFlat.update(m_CfdDriver->trapezoidFlat.get(channel));
	decayTime.update(m_CfdDriver->decayTime.get(channel));
	peaking.update(m_CfdDriver->peaking.get(channel));
	bitShift.update(m_CfdDriver->bitShift.get(channel));
}

void CAENCfdController::writeParam() {

	m_CfdDriver.execute(cfd::CfdDriver::WRITE_PARAMETERS_COMMAND, false);
}

void CAENCfdController::clearData() {
	commandProgression = 0;
//	m_CfdDriver.execute(cfd::CfdDriver::CLEAR_COMMAND, true);
	commandProgression = 100;
}

void CAENCfdController::initChannel(int32 aChannel) {

}

}
